@@ -142,10 +142,10 @@ class CLI(cl1.CLI):
 
 		# pull.
 		elif self.argument_present('--pull'):
-			path = self.get_argument("--pull", index=1)
-			remote = self.get_argument("--pull", index=2)
+			remote = self.get_argument("--pull", index=1)
+			path = self.get_argument("--pull", index=2)
 			if ":" not in remote:
-				r3sponse.log(error=f"Invalid <path> <alias>:<remote> format.", json=json)
+				r3sponse.log(error=f"Invalid <alias>:<remote> <path> format.", json=json)
 				self.stop()
 			alias,remote = remote.split(":")
 			exclude = None
@@ -164,8 +164,8 @@ class CLI(cl1.CLI):
 
 		# push.
 		elif self.argument_present('--push'):
-			remote = self.get_argument("--push", index=1)
-			path = self.get_argument("--push", index=2)
+			path = self.get_argument("--push", index=1)
+			remote = self.get_argument("--push", index=2)
 			if ":" not in remote:
 				r3sponse.log(error=f"Invalid <alias>:<remote> <path>.", json=json)
 				self.stop()
@@ -196,7 +196,6 @@ class CLI(cl1.CLI):
 				alias=alias, 
 				remote=remote, 
 				path=path,
-				delete=self.argument_present("--delete"),
 				forced=self.argument_present("--forced"), )
 			r3sponse.log(response=response, json=json)
 
