@@ -11,7 +11,8 @@ from ssht00ls.classes.agent import agent
 class SSync(object):
 	def __init__(self):
 		self.exclude = ['__pycache__', '.DS_Store']
-		aliases.synch()
+		aliases.sync()
+		self.sync()
 	def mount(self, 
 		# the local path.
 		path=None, 
@@ -25,6 +26,8 @@ class SSync(object):
 		exclude=['__pycache__', '.DS_Store'],
 		# accept new host verification keys.
 		accept_new_host_keys=True,
+		# log level.
+		log_level=0,
 	):
 
 		# checks.
@@ -40,6 +43,8 @@ class SSync(object):
 			return r3sponse.error_response(f"Path [{path}] already exists.")
 		elif "*mounted*" in str(cache.get(id=path, group="daemons")):
 			return r3sponse.error_response(f"Path [{path}] is already mounted.")
+		path = utils.clean_path(path)
+		remote = utils.clean_path(remote)
 
 		# pull.
 		response = self.pull(
@@ -49,7 +54,8 @@ class SSync(object):
 			exclude=exclude,
 			forced=forced,
 			delete=True,
-			safe=False,)
+			safe=False,
+			log_level=log_level,)
 		if not r3sponse.success: return response
 
 		# start daemon.
@@ -66,6 +72,8 @@ class SSync(object):
 		forced=False,
 		# sudo required.
 		sudo=False,
+		# log level.
+		log_level=0,
 	):
 
 		# checks.
@@ -80,6 +88,7 @@ class SSync(object):
 			return r3sponse.error_response(f"Path [{path}] is not a directory.")
 		elif "*mounted*" not in cache.get(id=path, group="daemons"):
 			return r3sponse.error_response(f"Path [{path}] is not mounted.")
+		path = utils.clean_path(path)
 
 		# wait for daemon stop.
 		response = self.stop_daemon(path=path)
@@ -98,11 +107,12 @@ class SSync(object):
 		return r3sponse.success_response(f"Successfully unmounted [{path}].")
 
 		#
-	def index(self, path=None, alias=None):
+	def index(self, path=None, alias=None, log_leve=0):
 
 		# checks.
 		if path == None:
 			return r3sponse.error_response(f"Define parameter: path.")
+		path = utils.clean_path(path)
 
 		# remote.
 		if alias != None:
@@ -153,10 +163,15 @@ class SSync(object):
 		safe=False,
 		# accept new hosts keys.
 		accept_new_host_keys=True,
+		# log level.
+		log_level=0,
 	):	
 		# check alias.
+		path = utils.clean_path(path)
+		remote = utils.clean_path(remote)
 		response = aliases.check(alias)
 		if not response.success: return response
+		
 		# check passphrase.
 		if CONFIG["aliases"][alias]["smart_card"] in [True, "true", "True"]:
 			response = ENCRYPTION.decrypt(CONFIG["aliases"][alias]["passphrase"])
@@ -164,6 +179,7 @@ class SSync(object):
 			response = ENCRYPTION.decrypt(CONFIG["aliases"][alias]["passphrase"])
 		if not response.success: return response
 		passphrase = response.decrypted.decode()
+		
 		# tests.
 		response = agent.add(path=CONFIG["aliases"][alias]["private_key"], passphrase=passphrase)
 		if not response["success"]: return response
@@ -171,18 +187,30 @@ class SSync(object):
 		if not response.success: return response
 		response = utils.test_ssht00ls(alias=alias, accept_new_host_keys=accept_new_host_keys)
 		if not response.success: return response
-		response = utils.test_ssh_path(alias=alias, path=path, accept_new_host_keys=accept_new_host_keys)
+		response = utils.test_ssh_path(alias=alias, path=remote, accept_new_host_keys=accept_new_host_keys)
 		if not response.success: return response
+
 		# pull.
 		if exclude == None: exclude = self.exclude
 		exclude_str = ""
 		for i in exclude: exclude_str += f"--exclude {i} "
 		delete_str = Formats.Boolean(delete).convert(true="--delete", false="")
 		command = f"rsync -azP {alias}:{remote} {path} {exclude_str}{delete_str}"
+		loader = syst3m.console.Loader(f"Pulling [{alias}:{remote}] to [{path}]")
 		output = syst3m.utils.__execute_script__(command)
+		if len(output) > 0 and output[len(output)-1] == "\n": output = output[:-1]
 		cache.set(id=path, data="*mounted*", group="daemons")
+		
 		# handler.
-		return r3sponse.success_response(f"Successfully pulled [{alias}:{remote}] to [{path}].")
+		if "rsync: " in output or "rsync error: " in output:
+			loader.stop(success=False)
+			print(output)
+			return r3sponse.success_response(f"Failed to pull [{alias}:{remote}] to [{path}].")
+		else:
+			loader.stop()
+			return r3sponse.success_response(f"Successfully pulled [{alias}:{remote}] to [{path}].")
+
+		#
 	def push(self,
 		# the local path.
 		path=None, 
@@ -200,10 +228,15 @@ class SSync(object):
 		safe=False,
 		# accept new hosts keys.
 		accept_new_host_keys=True,
+		# log level.
+		log_level=0,
 	):
 		# check alias.
+		path = utils.clean_path(path)
+		remote = utils.clean_path(remote)
 		response = aliases.check(alias)
 		if not response.success: return response
+		
 		# check passphrase.
 		if CONFIG["aliases"][alias]["smart_card"] in [True, "true", "True"]:
 			response = ENCRYPTION.decrypt(CONFIG["aliases"][alias]["passphrase"])
@@ -211,6 +244,7 @@ class SSync(object):
 			response = ENCRYPTION.decrypt(CONFIG["aliases"][alias]["passphrase"])
 		if not response.success: return response
 		passphrase = response.decrypted.decode()
+		
 		# tests.
 		response = agent.add(path=CONFIG["aliases"][alias]["private_key"], passphrase=passphrase)
 		if not response["success"]: return response
@@ -218,18 +252,35 @@ class SSync(object):
 		if not response.success: return response
 		response = utils.test_ssht00ls(alias=alias, accept_new_host_keys=accept_new_host_keys)
 		if not response.success: return response
-		response = utils.test_ssh_path(alias=alias, path=path, accept_new_host_keys=accept_new_host_keys)
+		response = utils.test_ssh_path(alias=alias, path=Formats.FilePath(remote).base(), accept_new_host_keys=accept_new_host_keys)
 		if not response.success: return response
+
 		# push.
 		if exclude == None: exclude = self.exclude
 		exclude_str = ""
 		for i in exclude: exclude_str += f"--exclude {i} "
 		delete_str = Formats.Boolean(delete).convert(true="--delete", false="")
 		command = f"rsync -azP {path} {alias}:{remote} {exclude_str}{delete_str}"
+		loader = syst3m.console.Loader(f"Pushing [{path}] to [{alias}:{remote}].")
 		output = syst3m.utils.__execute_script__(command)
+		if len(output) > 0 and output[len(output)-1] == "\n": output = output[:-1]
+		cache.set(id=path, data="*mounted*", group="daemons")
+		
 		# handler.
-		return r3sponse.success_response(f"Successfully pushed [{path}] to [{alias}:{remote}].")
+		if "rsync: " in output or "rsync error: " in output:
+			loader.stop(success=False)
+			print(output)
+			return r3sponse.success_response(f"Failed to push [{path}] to [{alias}:{remote}].")
+		else:
+			loader.stop()
+			return r3sponse.success_response(f"Successfully pushed [{path}] to [{alias}:{remote}].")
+
+
+		#
 	# the mounted daemon.
+	def sync(self):
+		for path in Files.Directory(path=f"{cache.path}/daemons/").paths(recursive=False):
+			a=1
 	def daemon(self, 
 		# the ssh alias.
 		alias=None, 
@@ -240,6 +291,8 @@ class SSync(object):
 		# settings.
 		start=True,
 	):
+		path = utils.clean_path(path)
+		remote = utils.clean_path(remote)
 		daemon = self.Daemon({
 			"alias":alias,
 			"remote":remote,
@@ -265,6 +318,7 @@ class SSync(object):
 		def __init__(self, attributes={}):
 			syst3m.objects.Thread.__init__(self)
 			self.assign(attributes)
+			self.path = utils.clean_path(self.path)
 			self.id = f"{self.alias}:{self.remote} ==> {self.path}"
 		def run(self):
 
@@ -294,6 +348,7 @@ class SSync(object):
 
 				# if file no longer exists stop daemon.
 				if not os.path.exists(self.path):
+					cache.set(id=self.path, data="*crashed*", group="daemons")
 					raise ValueError(f"SSHT00ls daemon ({self.id}): Mounted directory [{self.path}] from [{self.alias}:{self.remote}] no longer exists.")
 
 			# stop.
@@ -302,6 +357,12 @@ class SSync(object):
 # initialized objects.
 ssync = SSync()
 
+path = "/tmp/testmount"
+alias = "dev.vandenberghinc.com"
+response = ssync.index(path=path, alias=alias)
+print(response)
+quit()
+
 #print(ssync.mount(
 #	alias="dev.vandenberghinc.com",
 #	remote="/tmp/testmount",
